==> divine-4.0.22+2018.01.07.tar.gz: divine-4.0.22+2018.01.07/divine/mc/job.hpp <==
// -*- mode: C++; indent-tabs-mode: nil; c-basic-offset: 4 -*-

/*
 * (c) 2017 Petr Ročkai <code@fixp.eu>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#pragma once

#include <divine/ss/search.hpp>
#include <divine/vm/dbg-context.hpp>
#include <divine/vm/program.hpp>
#include <divine/mc/trace.hpp>
#include <brick-mem>
#include <brick-shmem>

namespace divine {
namespace mc {

enum class Result { None, Valid, Error, BootError };

static std::ostream &operator<<( std::ostream &o, Result r )
{
    switch ( r )
    {
        case Result::None: return o << "error found: unknown";
        case Result::Valid: return o << "error found: no";
        case Result::Error: return o << "error found: yes";
        case Result::BootError: return o << "error found: boot";
    }
}

struct Job : ss::Job
{
    std::shared_ptr< brick::shmem::ThreadBase > _monitor_loop;
    std::function< void( bool ) > _monitor;
    std::function< int64_t() > statecount = []() { return 0; };
    std::function< int64_t() > queuesize =  []() { return 0; };
    std::shared_ptr< ss::Job > _search;

    template< typename Monitor >
    void start( int threads, Monitor monit )
    {
        start( threads ); /* virtual */
        _monitor = monit;
    }

    void wait() override
    {
        auto clock = std::chrono::steady_clock::now();
        auto search = std::async( [&] { _search->wait(); } );
        auto cleanup = [&] { _search->stop(); if ( _monitor ) _monitor( true ); };
        using brick::shmem::wait;

        while ( wait( &search, &search + 1, cleanup, clock ) == std::future_status::timeout )
        {
            clock += std::chrono::milliseconds( 500 );
            if ( _monitor )
                try { _monitor( false ); } catch ( ... ) { cleanup(); throw; };
        }
    }

    void stop() override
    {
        _search->stop();
    }

    using PoolStats = std::map< std::string, brick::mem::Stats >;
    using DbgCtx = vm::dbg::Context< vm::CowHeap >;

    virtual Trace ce_trace() { return Trace(); }
    virtual Result result() { return Result::None; }
    virtual PoolStats poolstats() { return PoolStats(); }
    virtual void dbg_fill( DbgCtx & ) {}
    virtual void start( int ) override = 0;
};

template< template< typename, typename > class Job_, typename Next >
std::shared_ptr< Job > make_job( vm::explore::BC bc, Next next )
{
    if ( bc->is_symbolic() ) {
        auto solver = bc->solver();
        if ( solver == "z3" )
            return std::make_shared< Job_< Next, vm::Z3Explore > >( bc, next );
        else if ( solver == "boolector" )
            return std::make_shared< Job_< Next, vm::BoolectorExplore > >( bc, next );
        UNREACHABLE( "Using unsupported solver." );
    }
    return std::make_shared< Job_< Next, vm::Explore > >( bc, next );
}

}
}

==> divine-4.1.0+2018.01.19.tar.gz: divine-4.1.0+2018.01.19/divine/mc/job.hpp <==
// -*- mode: C++; indent-tabs-mode: nil; c-basic-offset: 4 -*-

/*
 * (c) 2017 Petr Ročkai <code@fixp.eu>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#pragma once

#include <divine/ss/search.hpp>
#include <divine/vm/dbg-context.hpp>
#include <divine/vm/program.hpp>
#include <divine/mc/trace.hpp>
#include <brick-mem>
#include <brick-shmem>

namespace divine {
namespace mc {

enum class Result { None, Valid, Error, BootError };

static std::ostream &operator<<( std::ostream &o, Result r )
{
    switch ( r )
    {
        case Result::None: return o << "error found: unknown";
        case Result::Valid: return o << "error found: no";
        case Result::Error: return o << "error found: yes";
        case Result::BootError: return o << "error found: boot";
    }
}

struct Job : ss::Job
{
    std::shared_ptr< brick::shmem::ThreadBase > _monitor_loop;
    std::function< void( bool ) > _monitor;
    std::function< std::pair< int64_t, int64_t >() > stats = []() { return std::make_pair( 0, 0 ); };
    std::function< int64_t() > queuesize = []() { return 0; };
    std::shared_ptr< ss::Job > _search;

    template< typename Monitor >
    void start( int threads, Monitor monit )
    {
        start( threads ); /* virtual */
        _monitor = monit;
    }

    void wait() override
    {
        auto clock = std::chrono::steady_clock::now();
        auto search = std::async( [&] { _search->wait(); } );
        auto cleanup = [&] { _search->stop(); if ( _monitor ) _monitor( true ); };
        using brick::shmem::wait;

        while ( wait( &search, &search + 1, cleanup, clock ) == std::future_status::timeout )
        {
            clock += std::chrono::milliseconds( 500 );
            if ( _monitor )
                try { _monitor( false ); } catch ( ... ) { cleanup(); throw; };
        }
    }

    void stop() override
    {
        _search->stop();
    }

    using PoolStats = std::map< std::string, brick::mem::Stats >;
    using DbgCtx = vm::dbg::Context< vm::CowHeap >;

    virtual Trace ce_trace() { return Trace(); }
    virtual Result result() { return Result::None; }
    virtual PoolStats poolstats() { return PoolStats(); }
    virtual void dbg_fill( DbgCtx & ) {}
    virtual void start( int ) override = 0;
};

template< template< typename, typename > class Job_, typename Next >
std::shared_ptr< Job > make_job( vm::explore::BC bc, Next next )
{
    if ( bc->is_symbolic() ) {
        auto solver = bc->solver();
        if ( solver == "z3" )
            return std::make_shared< Job_< Next, vm::Z3Explore > >( bc, next );
        else if ( solver == "boolector" )
            return std::make_shared< Job_< Next, vm::BoolectorExplore > >( bc, next );
        UNREACHABLE( "Using unsupported solver." );
    }
    return std::make_shared< Job_< Next, vm::ExplicitExplore > >( bc, next );
}

}
}
